/* 
 *  print_ioctl.c  --   a rudimentary decode of ioctl's 2nd arg
 *                      (built/tested under: Digital Unix V3.2)
 *
 *  John Hascall (john@iastate.edu)
 *  Iowa State University Computation Center
 *  5-Feb-1996
 *
 */
#include "trace.h"
#include "sysname.h"
#include <sys/socket.h>
#include <sys/ioctl.h>

extern FILE *outfp;
extern int max_string;


/* ioctl(fd, req, arg) */

int printargs_ioctl(
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int		fd	= sysArgs[0];
	unsigned long	req	= sysArgs[1];
	caddr_t		arg	= (caddr_t)sysArgs[2];
	unsigned int	group	= 0;
	unsigned int	num	= 0;
	char *		dir	= "?";
	char *		name	= "unknown";
	int		arglen	= 0;

	arglen = IOCPARM_LEN(req);
	switch (req & IOC_DIRMASK) {
	    case IOC_INOUT:	dir = "InOut";	break;
	    case IOC_IN:	dir = "In";	break;
	    case IOC_OUT:	dir = "Out";	break;
	    case IOC_VOID:	dir = "Void";	break;
	}
	group = (req >> 8) & 0xff;
	num = req & 0xff;
	switch (group) {
	    case 't':
		switch (num) {
		    case   0:	name = "TIOCGETD";	break;	/* compat */
		    case   1:	name = "TIOCSETD";	break;	/* compat */
		    case   2:	name = "TIOCHPCL";	break;	/* compat */
		    case   3:	name = "TIOCMODG";	break;
		    case   4:	name = "TIOCMODS";	break;
		    case   8:	name = "TIOCGETP";	break;	/* compat */
		    case   9:	name = "TIOCSETP";	break;	/* compat */
		    case  10:	name = "TIOCSETN";	break;	/* compat */
		    case  13:	name = "TIOCEXCL";	break;
		    case  14:	name = "TIOCNXCL";	break;
		    case  16:	name = "TIOCFLUSH";	break;
		    case  17:	name = "TIOCSETC";	break;	/* compat */
		    case  18:	name = "TIOCGETC";	break;	/* compat */
		    case  19:	name = "TIOCGETA";	break;
		    case  20:	name = "TIOCSETA";	break;
		    case  21:	name = "TIOCSETAW";	break;
		    case  22:	name = "TIOCSETAF";	break;
		    case  23:	name = "TCGETA";	break;
		    case  24:	name = "TCSETA";	break;
		    case  25:	name = "TCSETAW";	break;
		    case  26:	name = "TIOCGETD";	break;
		    case  27:	name = "TIOCSETD";	break;
		    case  28:	name = "TCSETAF";	break;
		    case  29:	name = "TCSBREAK";	break;
		    case  30:	name = "TCXONC";	break;
		    case  31:	name = "TCFLSH";	break;
		    case  71:	name = "ISPTM";		break;
		    case  72:	name = "TIOCGSID";	break;
		    case  73:	name = "UNLKPT";	break;
		    case  74:	name = "ISPTS";		break;
		    case  79:	name = "TIOCMODEM";	break;	/* compat */
		    case  80:	name = "TIOCNMODEM";	break;	/* compat */
		    case  88:	name = "SLIOGUNIT";	break;
		    case  89:	name = "SLIOCSFLAGS";	break;
		    case  90:	name = "SLIOCGFLAGS";	break;
		    case  95:	name = "TIOCSIG";	break;
		    case  96:	name = "TIOCEXT";	break;
		    case  97:	name = "TIOCSCTTY";	break;
		    case  98:	name = "TIOCCONS";	break;
		    case 100:	name = "TIOCCMLB";	break;	/* compat */
		    case 101:	name = "TIOCSMLB";	break;	/* compat */
		    case 102:	name = "TIOCUCNTL";	break;
		    case 103:	name = "TIOCSWINSZ";	break;
		    case 104:	name = "TIOCGWINSZ";	break;
		    case 105:	name = "TIOCREMOTE";	break;
		    case 106:	name = "TIOCMGET";	break;
		    case 107:	name = "TIOCMBIC";	break;
		    case 108:	name = "TIOCMBIS";	break;
		    case 109:	name = "TIOCMSET";	break;
		    case 110:	name = "TIOCSTART";	break;
		    case 111:	name = "TIOCSTOP";	break;
		    case 112:	name = "TIOCPKT";	break;
		    case 113:	name = "TIOCNOTTY";	break;
		    case 114:	name = "TIOCSTI";	break;
		    case 115:	name = "TIOCOUTQ";	break;
		    case 116:	name = "TIOCGLTC";	break;	/* compat */
		    case 117:	name = "TIOCSLTC";	break;	/* compat */
		    case 118:	name = "TIOCSPGRP";	break;
		    case 119:	name = "TIOCGPGRP";	break;
		    case 120:	name = "TIOCCDTR";	break;
		    case 121:	name = "TIOCSDTR";	break;
		    case 122:	name = "TIOCCBRK";	break;
		    case 123:	name = "TIOCSBRK";	break;
		    case 124:	name = "TIOCLGET";	break;	/* compat */
		    case 125:	name = "TIOCLSET";	break;	/* compat */
		    case 126:	name = "TIOCLBIC";	break;	/* compat */
		    case 127:	name = "TIOCLBIS";	break;	/* compat */
		    default:	name = "TIOCunknown";	break;
		}
		break;
	    case 'f':
		switch (num) {
		    case   1:	name = "FIOCLEX";	break;
		    case   2:	name = "FIONCLEX";	break;
		    case 120:	name = "FIOFDETACH";	break;
		    case 121:	name = "FIOFATTACH";	break;
		    case 122:	name = "FIOPIPESTAT";	break;
		    case 123:	name = "FIOGETOWN";	break;
		    case 124:	name = "FIOSETOWN";	break;
		    case 125:	name = "FIOASYNC";	break;
		    case 126:	name = "FIONBIO";	break;
		    case 127:	name = "FIONREAD";	break;
		    default:    name = "FIOunknown";	break;
		}
		break;
	    case 's':
		switch (num) {
			/* 0 = UAGT_CAM_IO */
			/* 1 = UAGT_CAM_SINGLE_SCAN */
			/* 2 = UAGT_CAM_FULL_SCAN */
		    case   0:	name = "SIOCSHIWAT";	break;
		    case   1:	name = "SIOCGHIWAT";	break;
		    case   2:	name = "SIOCSLOWAT";	break;
		    case   3:	name = "SIOCGLOWAT";	break;
		    case   7:	name = "SIOCATMARK";	break;
		    case   8:	name = "SIOCSPGRP";	break;
		    case   9:	name = "SIOCGPGRP";	break;
		    case  10:	name = "SIOCGPEERPRIV";	break;
		    default:    name = "SIOCunknown";	break;
		}
		break;
	    case 'r':
		switch (num) {
		    case  10:	name = "SIOCADDRT";	break;
		    case  11:	name = "SIOCDELRT";	break;
		    default:    name = "RTunknown";	break;
		}
		break;
	    case 'i':
		switch (num) {
		    case  12:	name = "SIOCSIFADDR";		break;
		    case  13:	name = "SIOCGIFADDR";		break;	/* O */
		    case  14:	name = "SIOCSIFDSTADDR";	break;
		    case  15:	name = "SIOCGIFDSTADDR";	break;	/* O */
		    case  16:	name = "SIOCSIFFLAGS";		break;
		    case  17:	name = "SIOCGIFFLAGS";		break;
		    case  18:	name = "SIOCGIFBRDADDR";	break;	/* O */
		    case  19:	name = "SIOCSIFBRDADDR";	break;
		    case  20:	name = "SIOCGIFCONF";		break;	/* O */
		    case  21:	name = "SIOCGIFNETMASK";	break;	/* O */
		    case  22:	name = "SIOCSIFNETMASK";	break;
		    case  23:	name = "SIOCGIFMETRIC";		break;
		    case  24:	name = "SIOCSIFMETRIC";		break;
		    case  25:	name = "SIOCDIFADDR";		break;
		    case  26:	name = "SIOCAIFADDR";		break;
		    case  27:	name = "SIOCRDZCTRS";		break;
		    case  28:	name = "SIOCRDCTRS";		break;
		    case  29:	name = "SIOCPIFADDR";		break;
		    case  30:	name = "SIOCSARP";		break;
		    case  31:	name = "SIOCGARP";		break;	/* O */
		    case  32:	name = "SIOCDARP";		break;
		    case  33:	name = "SIOCGIFADDR";		break;	/* N */
		    case  34:	name = "SIOCGIFDSTADDR";	break;	/* N */
		    case  35:	name = "SIOCGIFBRDADDR";	break;	/* N */
		    case  36:	name = "SIOCGIFCONF";		break;	/* N */
		    case  37:	name = "SIOCGIFNETMASK";	break;	/* N */
		    case  38:	name = "SIOCGARP";		break;	/* N */
		    case  40:	name = "SIOCARPREQ";		break;
		    case  45:	name = "SIOCMANREQ";		break;
		    case  46:	name = "SIOCGETEVENTS";		break;
		    case  47:	name = "SIOCIFRESET";		break;
		    case  48:	name = "SIOCEEUPDATE";		break;
		    case  49:	name = "SIOCADDMULTI";		break;
		    case  50:	name = "SIOCDELMULTI";		break;
		    case  51:	name = "SIOCSCREENON";		break;
		    case  52:	name = "SIOCSCREEN";		break;
		    case  53:	name = "SIOCSCREENSTATS";	break;
		    case  60:	name = "SIOCENABLBACK";		break;
		    case  61:	name = "SIOCDISABLBACK";	break;
		    case  62:	name = "SIOCRPHYSADDR";		break;
		    case  63:	name = "SIOCSPHYSADDR";		break;
		    case  64:	name = "SIOCIFSETCHAR";		break;
		    case  65:	name = "SIOCSMACSPEED";		break;
		    case  66:	name = "SIOCRMACSPEED";		break;
		    case  67:	name = "SIOCSIPMTU";		break;
		    case  68:	name = "SIOCRIPMTU";		break;
		    case  70:	name = "SIOCSRREQR";		break;
		    case  71:	name = "SIOCSRREQW";		break;
		    default:    name = "SIOxIFunknown";		break;	/* N */
		}
		break;
	    case 'm':
		switch (num) {
		    case   1:	name = "MTIOCTOP";	break;
		    case   2:	name = "MTIOCGET";	break;
		    default:    name = "MTIOCunknown";	break;
		}
		break;
	    case 'S':
		switch (num) {
		    case   1:	name = "I_NREAD";		break;
		    case   2:	name = "I_PUSH";		break;
		    case   3:	name = "I_POP";			break;
		    case   4:	name = "I_LOOK";		break;
		    case   5:	name = "I_FLUSH";		break;
		    case   6:	name = "I_SRDOPT";		break;
		    case   7:	name = "I_GRDOPT";		break;
		    case   8:	name = "I_STR";			break;
		    case   9:	name = "I_SETSIG";		break;
		    case  10:	name = "I_GETSIG";		break;
		    case  11:	name = "I_FIND";		break;
		    case  12:	name = "I_LINK";		break;
		    case  13:	name = "I_UNLINK";		break;
		    case  14:	name = "I_ISASTREAM";		break;
		    case  15:	name = "I_PEEK";		break;
		    case  16:	name = "I_FDINSERT";		break;
		    case  17:	name = "I_SENDFD";		break;
		    case  18:	name = "I_RECVFD";		break;
		    case  19:	name = "I_FLUSHBAND";		break;
		    case  20:	name = "I_SWROPT";		break;
		    case  21:	name = "I_GWROPT";		break;
		    case  22:	name = "I_LIST";		break;
		    case  23:	name = "I_ATMARK";		break;
		    case  24:	name = "I_CKBAND";		break;
		    case  25:	name = "I_GETBAND";		break;
		    case  26:	name = "I_CANPUT";		break;
		    case  27:	name = "I_SETCLTIME";		break;
		    case  28:	name = "I_GETCLTIME";		break;
		    case  29:	name = "I_PLINK";		break;
		    case  30:	name = "I_PUNLINK";		break;
		    case  40:	name = "I_GETMSG";		break;
		    case  41:	name = "I_PUTMSG";		break;
		    case  42:	name = "I_GETPMSG";		break;
		    case  43:	name = "I_PUTPMSG";		break;
		    case  44:	name = "I_PIPE";		break;
		    case  45:	name = "I_FIFO";		break;
		    case  60:	name = "I_STR_ATTR";		break;
		    case  61:	name = "I_PEEK_ATTR";		break;
		    case  62:	name = "I_FDINSERT_ATTR";	break;
		    case  63:	name = "I_SENDFD_ATTR";		break;
		    case  64:	name = "I_RECVFD_ATTR";		break;
		    case  65:	name = "I_GETMSG_ATTR";		break;
		    case  66:	name = "I_PUTMSG_ATTR";		break;
		    default:    name = "I_unknown";		break;
		}
		break;
	    case 'B':
		switch (num) {
		    case   1:	name = "BINLOG_ENABLE";		break;
		    case   2:	name = "BINLOG_DISABLE";	break;
		    case   3:	name = "BINLOG_ASCIIENABLE";	break;
		    case   4:	name = "BINLOG_ASCIIDISABLE";	break;
		    case   5:	name = "BINLOG_CLRCNTRS";	break;
		    case   6:	name = "BINLOG_SETPID";		break;
		    case  20:	name = "BINLOG_GETSTATUS";	break;
		    default:    name = "BINLOG_unknown";	break;
		}
		break;
	    case 'v':
		switch (num) {
		    case   1:	name = "DEVIOCGET";	break;	/* compat */
		    case   2:	name = "DEVGETGEOM";	break;	/* compat */
		    case   3:	name = "SRVC_REQUEST";	break;	/* compat */
		    case   4:	name = "DEVROOT";	break;	/* compat */
		    case   5:	name = "DIODCMD";	break;	/* compat */
		    case   6:	name = "DIODCDB";	break;	/* compat */
		    default:	name = "DEVunknown";	break;	/* compat */
		}
		break;
	    case 'p':
		switch (num) {
		    case   1:	name = "DIOCGETPT";		break;
		    case   2:	name = "DIOCSETPT";		break;
		    case   3:	name = "DIOCDGTPT";		break;
		    case   4:   name = "SCSI_FORMAT_UNIT";	break;
		    case   5:   name = "SCSI_REASSIGN_BLOCK";	break;
		    case   6:   name = "SCSI_READ_DEFECT_DATA";	break;
		    case   7:   name = "SCSI_VERIFY_DATA";	break;
		    case   8:   name = "SCSI_MODE_SELECT";	break;
		    case   9:   name = "SCSI_MODE_SENSE";	break;
		    case  10:   name = "SCSI_GET_SENSE";	break;
		    case  11:   name = "SCSI_GET_INQUIRY_DATA";	break;
		    case  12:   name = "SCSI_READ_LONG";	break;
		    case  13:   name = "SCSI_WRITE_LONG";	break;
		    case  14:   name = "SCSI_READ_CAPACITY";	break;
		    case  15:   name = "SCSI_STOP_UNIT";	break;
		    case  16:   name = "SCSI_START_UNIT";	break;
		    case  17:   name = "SCSI_ALLOW_REMOVAL";	break;
		    case  18:   name = "SCSI_PREVENT_REMOVAL";	break;
		    case  19:   name = "SCSI_SEEK_POSITION";	break;
		    case  20:   name = "SCSI_LOAD_UNIT";	break;
		    case  21:   name = "SCSI_UNLOAD_UNIT";	break;
		    case  22:   name = "SCSI_SEND_DIAGNOSTIC";	break;
		    case  23:   name = "SCSI_RECEIVE_DIAGNOSTIC";	break;
		    case  24:   name = "SCSI_READ_BUFFER";	break;
		    case  25:   name = "SCSI_WRITE_BUFFER/CDROM_PAUSE_PLAY";	break;
		    case  26:	name = "CDROM_RESUME_PLAY";	break;
		    case  27:	name = "CDROM_PLAY_AUDIO";	break;
		    case  28:	name = "CDROM_PLAY_AUDIO_MSF";	break;
		    case  29:	name = "CDROM_PLAY_AUDIO_TI";	break;
		    case  30:	name = "CDROM_PLAY_AUDIO_TR";	break;
		    case  31:	name = "CDROM_TOC_HEADER";	break;
		    case  32:	name = "CDROM_TOC_ENTRYS";	break;
		    case  33:	name = "CDROM_EJECT_CADDY";	break;
		    case  36:	name = "CDROM_READ_SUBCHANNEL";	break;
		    case  38:	name = "CDROM_GET_DATAMODE";	break;
		    case  39:	name = "CDROM_SET_DATAMODE";	break;
		    case  40:	name = "CDROM_READ_HEADER";	break;
		    case  50:	name = "CDROM_PLAY_VAUDIO";	break;
		    case  51:	name = "CDROM_PLAY_MSF";	break;
		    case  52:	name = "CDROM_PLAY_TRACK";	break;
		    case  53:	name = "CDROM_PLAYBACK_CONTROL";break;
		    case  54:	name = "CDROM_PLAYBACK_STATUS";	break;
		    case  55:	name = "CDROM_SET_ADDRESS_FORMAT";	break;
		    case 100:   name = "SCSI_SPECIAL";		break;
		    case 128:	name = "SCSI_INIT_ELEMENT";	break;
		    case 129:	name = "SCSI_POSITION_ELEMENT";	break;
		    case 130:	name = "SCSI_MOVE_MEDIUM";	break;
		    case 131:	name = "SCSI_EXCHANGE_MEDIUM";	break;
		    case 132:	name = "SCSI_READ_ELEMENT";	break;
		    case 133:	name = "SCSI_RESERVE_ELEMENT";	break;
		    case 134:	name = "SCSI_RELEASE_ELEMENT";	break;
		    case 135:	name = "SCSI_LOG_SENSE";	break;
		    case 136:	name = "SCSI_REQUEST_VOLUME_ELEMENT_ADDRESS";	break;
		    case 137:	name = "SCSI_SEND_VOLUME_TAG";	break;
		    case 139:	name = "SCSI_MEDIA_CHANGER_OP";	break;
		    case 140:	name = "MCIOC_INITIALIZE";	break;
		    case 141:	name = "MCIOC_FETCH_VOLUME";	break;
		    case 142:	name = "MCIOC_STORE_VOLUME";	break;
		    case 143:	name = "MCIOC_DETECT_STATE";	break;
		    case 144:	name = "MCIOC_DETECT_RANGE";	break;
		    case 145:	name = "ODET_RDY_IOD";		break;
		    default:	name = "DIOC/SCSI/CDROM/unknown";	break;
		}
		break;
	    case 'd':
		switch (num) {
		    case   1:	name = "DKIOCHDR";	break;	/* compat */
		    case   2:	name = "DKIOCDOP";	break;	/* compat */
		    case   3:	name = "DKIOCGET";	break;	/* compat */
		    case   4:	name = "DKIOCACC";	break;	/* compat */
		    case   5:	name = "DKIOCEXCL";	break;	/* compat */
		    default:	name = "DKIOunknown";	break;	/* compat */
		}
		break;
	}

	fprintf(outfp, "ioctl (%d, 0x%08.8x<%s,%s,%u>, %p) = ",
	    fd, req, dir, name, arglen, arg);
	return(1);
}

int printret_ioctl(
	long	sysCallno,
	long	sysReturn,
	int	sysErrno
) {
	if (sysReturn != -1) {
		fprintf(outfp, "%d\n", (int)sysReturn);
	} else {
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
	}
	return (1);
} 
