/* 
 *  print_pid_block.c -	pid_block, pid_unblock - stops (blocks) or 
 *       resumes (unblocks) the specified process.
 *       Print arguments and return values for pid_block and pid_unblock
 *
 *  <tanmaykm@eureka.ad.infosys.com> 
 *  Fri Aug 24 15:12:51 GMT-5 2001
 *
 */
#include "trace.h"
#include "sysname.h"


extern FILE *outfp;
extern int max_string;

#include <sys/time.h>

/* --------------------------- public routines --------------------------- */

/*
 * called on syscall entry
 */
int printargs_pid_block(long sysCallno, long *sysArgs, int sysNumargs, char *sysFormat) 
{
    fprintf(outfp,"pid_block (");
	pr_timeval((struct timeval *)sysArgs[0]);
	fprintf(outfp,", %ld) = ", (long)sysArgs[1]);

	return (1);
}


/*
 * called on syscall exit
 */
int printret_pid_block(long sysCallno, long sysReturn, int sysErrno) 
{
	if (sysReturn == -1) 
	{
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d\n", (int)sysReturn);
	return (1);
}

/*
 * called on syscall entry
 */
int printargs_pid_unblock(long sysCallno, long *sysArgs, int sysNumargs, char *sysFormat) 
{
    fprintf(outfp,"pid_unblock (%ld, %ld) = ", (pid_t)sysArgs[0], (long)sysArgs[1]);
	return (1);
}


/*
 * called on syscall exit
 */
int printret_pid_unblock(long sysCallno, long sysReturn, int sysErrno) 
{
	if (sysReturn == -1) 
	{
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d\n", (int)sysReturn);
	return (1);
}

/* --------------------------- private routines --------------------------- */

