/* 
 *  print_rlimit.c -	Decode getrlimit/setrlimit
 *
 *  <john@ratbert.cc.iastate.edu> John Hascall
 *  Sun Mar  3 00:10:01 CST 1996
 *
 */
#include "trace.h"
#include "sysname.h"
#include <sys/time.h>
#include <sys/resource.h>


extern FILE *	outfp;
extern int	max_string;


static long	savArgs[2];
static int	savNumargs;


int printargs_rlimit (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int		get	= (sysCallno == SYS_getrlimit);
	int		res	= (int)sysArgs[0];
	struct rlimit *	rlp	= (struct rlimit *)sysArgs[1];

	fprintf(outfp, "%cetrlimit (", get ? 'g' : 's');
	switch (res) {
	    case RLIMIT_CPU:	fprintf(outfp, "RLIMIT_CPU, ");		break;
	    case RLIMIT_FSIZE:	fprintf(outfp, "RLIMIT_FSIZE, ");	break;
	    case RLIMIT_DATA:	fprintf(outfp, "RLIMIT_DATA, ");	break;
	    case RLIMIT_STACK:	fprintf(outfp, "RLIMIT_STACK, ");	break;
	    case RLIMIT_CORE:	fprintf(outfp, "RLIMIT_CORE, ");	break;
	    case RLIMIT_RSS:	fprintf(outfp, "RLIMIT_RSS, ");		break;
	    case RLIMIT_NOFILE:	fprintf(outfp, "RLIMIT_NOFILE, ");	break;
	    case RLIMIT_AS:	fprintf(outfp, "RLIMIT_AS, ");		break;
	    default:		fprintf(outfp, "RLIMIT_?#%u, ", res);
	}
	if (get) {
		savArgs[1] = sysArgs[1];
		fprintf(outfp, "0x%p) = ", rlp);
		return (1);
	}
	fprintf(outfp, "0x%p={", rlp);
	rlp = (struct rlimit *)rd_buf(sysArgs[1], sizeof(struct rlimit));
	if (rlp->rlim_cur == RLIM_INFINITY) {
		fprintf(outfp, "RLIM_INFINITY,");
	} else {
		fprintf(outfp, "%lu,", rlp->rlim_cur);
	}
	if (rlp->rlim_max == RLIM_INFINITY) {
		fprintf(outfp, "RLIM_INFINITY}) =");
	} else {
		fprintf(outfp, "%lu}) =", rlp->rlim_max);
	}
	free(rlp);
	return (1);
}


int printret_rlimit (
	long	sysCallno,
	long	sysReturn,
	int	sysErrno
) {
	int		get	= (sysCallno == SYS_getrlimit);
	struct rlimit *	rlp;

	if (sysReturn == -1) {
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d", (int)sysReturn);

	if (!get) {
		fprintf(outfp, "\n");
		return (1);
	}

	fprintf(outfp, " [ , {");
	rlp = (struct rlimit *)rd_buf(savArgs[1], sizeof(struct rlimit));
	if (rlp->rlim_cur == RLIM_INFINITY) {
		fprintf(outfp, "RLIM_INFINITY,");
	} else {
		fprintf(outfp, "%lu,", rlp->rlim_cur);
	}
	if (rlp->rlim_max == RLIM_INFINITY) {
		fprintf(outfp, "RLIM_INFINITY} ]\n");
	} else {
		fprintf(outfp, "%lu} ]\n", rlp->rlim_max);
	}
	free(rlp);
	return (1);
}
