/* 
 *  print_rusage.c -	Decode getrusage
 *
 *  <john@ratbert.cc.iastate.edu> John Hascall
 *  Sun Mar  3 01:32:46 CST 1996
 *
 */
#include "trace.h"
#include "sysname.h"
#include <sys/time.h>
#include <sys/resource.h>


/*
 * printargs_rusage uses these to pass args to
 * printret_rusage so it can display output args
 */
static long	savArgs[2];		/* XXX: size as needed */
static int	savNumargs;


extern FILE *	outfp;
extern int	max_string;


/* --------------------------- private routines --------------------------- */

/*
 * called on syscall entry
 */
int printargs_rusage (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int		who	= (int)sysArgs[0];
	struct rusage *	rup	= (struct rusage *)sysArgs[1];

	savArgs[1] = sysArgs[1];
	fprintf(outfp, "getrusage (RUSAGE_%s, 0x%p) = ",
	    (who == RUSAGE_SELF) ? "SELF" : "CHILDREN", rup);
	return (1);
}


/*
 * called on syscall exit
 */
int printret_rusage (
	long	sysCallno,
	long	sysReturn,
	int	sysErrno
) {
	struct rusage *	rup;

	if (sysReturn == -1) {
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d", (int)sysReturn);

	rup = (struct rusage *)rd_buf(savArgs[1], sizeof(struct rusage));
	fprintf(outfp, " [ , {");
	fprintf(outfp, "{%u,%u}, ", rup->ru_utime.tv_sec,rup->ru_utime.tv_usec);
	fprintf(outfp, "{%u,%u}, ", rup->ru_stime.tv_sec,rup->ru_stime.tv_usec);
	fprintf(outfp, "%lu, ", rup->ru_maxrss);
	fprintf(outfp, "%lu, ", rup->ru_ixrss);
	fprintf(outfp, "%lu, ", rup->ru_idrss);
	fprintf(outfp, "%lu, ", rup->ru_isrss);
	fprintf(outfp, "%lu, ", rup->ru_minflt);
	fprintf(outfp, "%lu, ", rup->ru_majflt);
	fprintf(outfp, "%lu, ", rup->ru_nswap);
	fprintf(outfp, "%lu, ", rup->ru_inblock);
	fprintf(outfp, "%lu, ", rup->ru_oublock);
	fprintf(outfp, "%lu, ", rup->ru_msgsnd);
	fprintf(outfp, "%lu, ", rup->ru_msgrcv);
	fprintf(outfp, "%lu, ", rup->ru_nsignals);
	fprintf(outfp, "%lu, ", rup->ru_nvcsw);
	fprintf(outfp, "%lu} ", rup->ru_nivcsw);
	fprintf(outfp, "]\n");
	free(rup);
	return (1);
}

/* --------------------------- private routines --------------------------- */

