/* 
 *  print_select.c -	Decode select
 *
 *  <john@ratbert.cc.iastate.edu> John Hascall
 *  Sun Mar  3 16:56:58 CST 1996
 *
 */
#include "trace.h"
#include "sysname.h"
#include "util.h"
#include <sys/types.h>
#include <sys/times.h>


/*
 * printargs_select uses these to pass args to
 * printret_select so it can display output args
 */
static long	savArgs[5];
static int	savNumargs;


static void	pr_fdset();

extern FILE *	outfp;
extern int	max_string;


/* --------------------------- public routines --------------------------- */

/*
 * called on syscall entry
 */
int printargs_select (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int	nfds	= (int)sysArgs[0];

	savArgs[0] = sysArgs[0];
	savArgs[1] = sysArgs[1];
	savArgs[2] = sysArgs[2];
	savArgs[3] = sysArgs[3];

	fprintf(outfp, "select (%d", nfds);

	fprintf(outfp, ", 0x%p=", (fd_set *)sysArgs[1]);
	pr_fdset(nfds, (fd_set *)sysArgs[1]);

	fprintf(outfp, ", 0x%p=", (fd_set *)sysArgs[2]);
	pr_fdset(nfds, (fd_set *)sysArgs[2]);

	fprintf(outfp, ", 0x%p=", (fd_set *)sysArgs[3]);
	pr_fdset(nfds, (fd_set *)sysArgs[3]);

	fprintf(outfp, ", 0x%p=", (fd_set *)sysArgs[1]);
	pr_timeval((struct timeval *)sysArgs[4]);

	fprintf(outfp, ") = ");
	return (1);
}


/*
 * called on syscall exit
 */
int printret_select (
	long	sysCallno,
	long	sysReturn,
	int	sysErrno
) {
	int	nfds	= (int)savArgs[0];

	if (sysReturn == -1) {
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d [ , ", (int)sysReturn);

	pr_fdset(nfds, (fd_set *)savArgs[1]);
	fprintf(outfp, ", ");

	pr_fdset(nfds, (fd_set *)savArgs[2]);
	fprintf(outfp, ", ");

	pr_fdset(nfds, (fd_set *)savArgs[3]);
	fprintf(outfp, ", ]\n");

	return (1);
}

/* --------------------------- private routines --------------------------- */


/*
 * this is pretty yucky, but...
 */

static void pr_fdset (
	int		nfds,
	struct fd_set *	fdset
) {
	int	i;
	int *	set;

	fprintf(outfp, "{");
	if (fdset == NULL) nfds = 0;
	for (i = 0; i < nfds; i += NFDBITS) {
		if (i != 0) fprintf(outfp, ",");
		set = fdset->fds_bits + i;
		set = (int *)rd_buf(set, sizeof(int));
		fprintf(outfp, "0x%08.8x", *set);
		free(set);
	}
	fprintf(outfp, "}");
}
