/* 
 *  print_sockaddr.c -	decode sockaddr oriented syscalls
 *			for popular address families
 *
 *  John Hascall (john@iastate.edu)
 *  Iowa State University Computation Center
 *  29-Feb-1996
 *
 */

/* XXX: need to handle nsyscall() variants!!! (len-byte hack!) */

#include "trace.h"
#include "sysname.h"
#include "util.h"

static long	savArgs[3];		/* to show results in exit-rtn */
static int	savNumargs;

static int	printexit_sockaddr();

extern FILE *	outfp;
extern int	max_string;

/*      accept(fd, name, namelenptr) */
/*        bind(fd, name, namelen)    */
/*     connect(fd, name, namelen)    */
/* getpeername(fd, name, namelenptr) */
/* getsockname(fd, name, namelenptr) */

int printargs_sockaddr(
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int			fd	= (int)sysArgs[0];
	struct sockaddr *	name;
	int *			namelenptr;
	int			namelen;
	int			inonly	= 0;
	int			bsd44	= 0;
	char *			which;

	switch (sysCallno) {
	    case SYS_naccept:       bsd44 = 1;
	    case SYS_accept:			which = "accept";	break;
	    case SYS_bind:         inonly = 1;	which = "bind";		break;
	    case SYS_connect:      inonly = 1;	which = "connect";	break;
	    case SYS_ngetpeername:  bsd44 = 1;
	    case SYS_getpeername:		which = "getpeername";	break;
	    case SYS_ngetsockname:  bsd44 = 1;
	    case SYS_getsockname:		which = "getsockname";	break;
	}
	if (inonly) {
		fprintf(outfp, "%s (%d, 0x%p=", which, fd, sysArgs[1]);
		namelen = (int)sysArgs[2];
		name	= (struct sockaddr *) rd_buf(sysArgs[1], namelen);
		pr_sockaddr(name, namelen, bsd44);
		fprintf(outfp, ", %d) = ", namelen);
		free(name);
		return(1);
	}
	fprintf(outfp, "%s (%d, 0x%p, 0x%p=", which, fd, sysArgs[1],sysArgs[2]);
	namelenptr = (int *) rd_buf(sysArgs[2], sizeof(int *));
	fprintf(outfp, "%d) = ", *namelenptr);
	free(namelenptr);
	savArgs[1] = sysArgs[1];
	savArgs[2] = sysArgs[2];
	return (1);
}


int printret_sockaddr (
	long	sysCallno,
	long	sysReturn,
	int	sysErrno
) {
	if (sysReturn == -1) {
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d", (int)sysReturn);
	if ((sysCallno == SYS_accept)      || (sysCallno == SYS_naccept) ||
	    (sysCallno == SYS_getpeername) || (sysCallno == SYS_ngetpeername) ||
	    (sysCallno == SYS_getsockname) || (sysCallno == SYS_ngetsockname)) {
		printexit_sockaddr(sysCallno, savArgs, savNumargs, "");
	}
	fprintf(outfp, "\n");
	return (1);
}


/* -------------------- private routines -------------------------- */

static int printexit_sockaddr (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	struct sockaddr *	name;
	int *			namelenptr;
	int			namelen;
	int			bsd44	= 0;

	if ((sysCallno == SYS_naccept) ||
	    (sysCallno == SYS_ngetpeername) ||
	    (sysCallno == SYS_ngetsockname)) {
		bsd44 = 1;
	}
	fprintf(outfp, " [ , ");
	namelenptr = (int *) rd_buf(sysArgs[2], sizeof(int *));
	namelen = *namelenptr;
	name = (struct sockaddr *) rd_buf(sysArgs[1], namelen);
	pr_sockaddr(name, namelen, bsd44);
	fprintf(outfp, ", %d ]", namelen);
	free(namelenptr);
	free(name);
	return (1);
}
