/* 
 *  print_sockio.c -	Decode recv, recvfrom, recvmsg, send, sendto, sendmsg
 *
 *  <john@ratbert.cc.iastate.edu> John Hascall
 *  Sun Mar  3 21:29:12 CST 1996
 *
 */
#include "trace.h"
#include "sysname.h"
#include <sys/types.h>
#include <sys/socket.h>


/*
 * printargs_sockio uses these to pass args to
 * printret_sockio so it can display output args
 */
static long	savArgs[6];
static int	savNumargs;


extern FILE *	outfp;
extern int	max_string;


/* --------------------------- public routines --------------------------- */

/*
 *     recv(socket, buffer, length, flags)
 *     send(socket, buffer, length, flags)
 * recvfrom(socket, buffer, length, flags, sockaddr, lenptr)
 *   sendto(socket, buffer, length, flags, sockaddr, len)
 *  recvmsg(socket, message, flags)
 *  sendmsg(socket, message, flags)
 */
int printargs_sockio (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	struct sockaddr *	sa;
	int			flags;
	int *			len;
	int			bsd44	= 0;

	savArgs[1] = sysArgs[1];
	switch (sysCallno) {
	    case SYS_recv:
		flags = (int)sysArgs[3];
		fprintf(outfp, "recv (%d, 0x%p, %d, ",
		    (int)sysArgs[0], sysArgs[1], sysArgs[2]);
		pr_flags(flags, MSG_PEEK, "MSG_PEEK", MSG_OOB,  "MSG_OOB", 0);
		fprintf(outfp, ") = ");
		return (1);
	    case SYS_nrecvfrom:
		bsd44 = 1;
		/*FALLTHROUGH*/
	    case SYS_recvfrom:
		flags = (int)sysArgs[3];
		savArgs[4] = sysArgs[4];
		savArgs[5] = sysArgs[5];
		fprintf(outfp, "recvfrom (%d, 0x%p, %d, ",
		    (int)sysArgs[0], sysArgs[1], sysArgs[2]);
		pr_flags(flags, MSG_PEEK, "MSG_PEEK", MSG_OOB,  "MSG_OOB", 0);
		len = (int *)rd_buf(sysArgs[5], sizeof(int));
		fprintf(outfp, ", 0x%p, 0x%p=%d) = ",
		    sysArgs[4], sysArgs[5], *len);
		free(len);
		return (1);
	    case SYS_nrecvmsg:
		bsd44 = 1;
		/*FALLTHROUGH*/
	    case SYS_recvmsg:
		flags = (int)sysArgs[3];
		fprintf(outfp, "recvmsg (%d, 0x%p, ",
		    (int)sysArgs[0], sysArgs[1]);
		pr_flags(flags, MSG_PEEK, "MSG_PEEK", MSG_OOB,  "MSG_OOB", 0);
		fprintf(outfp, ") = ");
		return (1);
	    case SYS_send:
		flags = (int)sysArgs[3];
		fprintf(outfp, "send (%d, 0x%p=", (int)sysArgs[0], sysArgs[1]);
		p_arg('s', sysArgs[1], 255);
		fprintf(outfp, "%d, ", sysArgs[2]);
		pr_flags(flags, MSG_PEEK, "MSG_PEEK", MSG_OOB,  "MSG_OOB", 0);
		fprintf(outfp, ") = ");
		return (1);
	    case SYS_sendto:
		flags = (int)sysArgs[3];
		fprintf(outfp, "sendto (%d, 0x%p=", (int)sysArgs[0],sysArgs[1]);
		p_arg('s', sysArgs[1], 255);
		fprintf(outfp, "p, %d, ", sysArgs[2]);
		pr_flags(flags, MSG_PEEK, "MSG_PEEK", MSG_OOB,  "MSG_OOB", 0);
/*
		pr_sockaddr();
*/
		fprintf(outfp, "%d) = ", (int)sysArgs[5]);
		return (1);
	    case SYS_nsendmsg:
		bsd44 = 1;
		/*FALLTHROUGH*/
	    case SYS_sendmsg:
		return (1);
	}
}


/*
 * called on syscall exit
 */
int printret_sockio (
	long	sysCallno,
	long	sysReturn,
	int	sysErrno
) {
	int	bsd44	= 0;

	if (sysReturn == -1) {
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d", (int)sysReturn);

	switch (sysCallno) {
	    case SYS_recv:
		break;
	    case SYS_nrecvfrom:
		bsd44 = 1;
	    case SYS_recvfrom:
		break;
	    case SYS_nrecvmsg:
		bsd44 = 1;
	    case SYS_recvmsg:
		break;
	    case SYS_send:
		break;
	    case SYS_sendto:
		break;
	    case SYS_nsendmsg:
		bsd44 = 1;
	    case SYS_sendmsg:
		break;
	}

	fprintf(outfp, "\n");
	return (1);
}

/* --------------------------- private routines --------------------------- */

