/* 
 *  print_sockopt.c -	Decode getsockopt/setsockopt requests
 *
 *  <john@ratbert.cc.iastate.edu> John Hascall
 *  Sat Mar  2 02:43:49 CST 1996
 *
 */
#include "trace.h"
#include "sysname.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>


static long	savArgs[5];
static int	savNumargs;

static int	printargs_ipopt();
static int	printargs_udpopt();
static int	printargs_tcpopt();

extern FILE *	outfp;
extern int	max_string;

/* getsockopt(socket, level, option, valueptr, lenptr) */
/* setsockopt(socket, level, option, valueptr, len) */

int printargs_sockopt (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int		level	= (int)sysArgs[1];
	int		opt	= (int)sysArgs[2];
	int		get	= (sysCallno == SYS_getsockopt);
	int *		len;
	int *		val;
	struct linger *	lv;

	savArgs[1] = sysArgs[1];		/* level */
	savArgs[2] = sysArgs[2];		/* option */
	savArgs[3] = sysArgs[3];		/* value address */
	savArgs[4] = sysArgs[4];		/* value len[ptr] */

	switch (level) {
	    case SOL_SOCKET:
		break;
	    case IPPROTO_IP:
		printargs_ipopt(sysCallno, sysArgs, sysNumargs, sysFormat);
		return (1);
	    case IPPROTO_TCP:
		printargs_tcpopt(sysCallno, sysArgs, sysNumargs, sysFormat);
		return (1);
	    case IPPROTO_UDP:
		printargs_udpopt(sysCallno, sysArgs, sysNumargs, sysFormat);
		return (1);
	    default:
		printargs_gen(sysCallno, sysArgs, sysNumargs, sysFormat);
		return (1);
	}
	fprintf(outfp, "%cetsockopt (%d, SOL_SOCKET, ",
	    get ? 'g' : 's', (int)sysArgs[0]);
	switch (opt) {
	    case SO_DEBUG:	fprintf(outfp, "SO_DEBUG, ");		break;
	    case SO_ACCEPTCONN:	fprintf(outfp, "SO_ACCEPTCONN, ");	break;
	    case SO_REUSEADDR:	fprintf(outfp, "SO_REUSEADDR, ");	break;
	    case SO_KEEPALIVE:	fprintf(outfp, "SO_KEEPALIVE, ");	break;
	    case SO_DONTROUTE:	fprintf(outfp, "SO_DONTROUTE, ");	break;
	    case SO_BROADCAST:	fprintf(outfp, "SO_BROADCAST, ");	break;
	    case SO_USELOOPBACK:fprintf(outfp, "SO_USELOOPBACK, ");	break;
	    case SO_LINGER:	fprintf(outfp, "SO_LINGER, ");		break;
	    case SO_OOBINLINE:	fprintf(outfp, "SO_OOBINLINE, ");	break;
	    case SO_REUSEPORT:	fprintf(outfp, "SO_REUSEPORT, ");	break;
	    case SO_SNDBUF:	fprintf(outfp, "SO_SNDBUF, ");		break;
	    case SO_RCVBUF:	fprintf(outfp, "SO_RCVBUF, ");		break;
	    case SO_SNDLOWAT:	fprintf(outfp, "SO_SNDLOWAT, ");	break;
	    case SO_RCVLOWAT:	fprintf(outfp, "SO_RCVLOWAT, ");	break;
	    case SO_SNDTIMEO:	fprintf(outfp, "SO_SNDTIMEO, ");	break;
	    case SO_RCVTIMEO:	fprintf(outfp, "SO_RCVTIMEO, ");	break;
	    case SO_ERROR:	fprintf(outfp, "SO_ERROR, ");		break;
	    case SO_TYPE:	fprintf(outfp, "SO_TYPE, ");		break;
	    default:		fprintf(outfp, "SO_?#%d, ", opt);	break;
	}
	if (get) {
		len = (int *)rd_buf(sysArgs[4], sizeof(int));
		fprintf(outfp, "0x%p, 0x%p=%u) = ",
		    sysArgs[3], sysArgs[4], *len);
		free(len);
		return (1);
	}
	fprintf(outfp, "0x%p=", sysArgs[3]);
	if (opt == SO_LINGER) {
		lv = (struct linger *)rd_buf(sysArgs[3], sizeof(struct linger));
		fprintf(outfp, "{%u,%u}", lv->l_onoff, lv->l_linger);
		free(lv);
	} else {
		val = (int *)rd_buf(sysArgs[3], sizeof(int));
		fprintf(outfp, "%u", *val);
		free(val);
	}
	fprintf(outfp, ", %u) = ", (int)sysArgs[4]);
	return (1);
}

int printret_sockopt (
	long	sysCallno,
	long	sysReturn,
	int	sysErrno
) {
	int		level	= (int)savArgs[1];
	int		opt	= (int)savArgs[2];
	int *		len;
	int *		val;

	if (sysReturn == -1) {
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d", (int)sysReturn);

	if ((sysCallno == SYS_setsockopt) ||
	    ((level != SOL_SOCKET) && (level != IPPROTO_TCP))) {
		fprintf(outfp, "\n");
		return (1);
	}
	if ((level == SOL_SOCKET) && (opt == SO_LINGER)) 
	{
		struct linger *	lv;
		lv = (struct linger *)rd_buf(savArgs[3], sizeof(struct linger));
		len = (int *)rd_buf(savArgs[4], sizeof(int));
		fprintf(outfp, " [ , , , {%u,%u}, %u ]\n",
		    lv->l_onoff, lv->l_linger, *len);
		free(len);
		free(lv);
		return (1);
	}
	val = (int *)rd_buf(savArgs[3], sizeof(int));
	len = (int *)rd_buf(savArgs[4], sizeof(int));
	fprintf(outfp, " [ , , , %u, %u ]\n", *val, *len);
	free(len);
	return (1);
}

/* ---------------------------- private routines --------------------------- */

static int printargs_ipopt (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int	level	= (int)sysArgs[1];
	int	opt	= (int)sysArgs[2];
	int	get	= (sysCallno == SYS_getsockopt);

	/*
	 * too yucky to think about for now...
	 */
	printargs_gen(sysCallno, sysArgs, sysNumargs, sysFormat);
	return (1);
}

static int printargs_udpopt (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int	level	= (int)sysArgs[1];
	int	opt	= (int)sysArgs[2];
	int	get	= (sysCallno == SYS_getsockopt);

	/*
	 * as far as I know, no such options exist, but here's
	 * a place to decode them if they (ever) do exist
	 */
	printargs_gen(sysCallno, sysArgs, sysNumargs, sysFormat);
	return (1);
}

static int printargs_tcpopt (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	int	level	= (int)sysArgs[1];
	int	opt	= (int)sysArgs[2];
	int	get	= (sysCallno == SYS_getsockopt);
	int *	len;
	int *	val;

	fprintf(outfp, "%cetsockopt (%d, IPPROTO_TCP, ",
	    get ? 'g' : 's', (int)sysArgs[0]);
	switch (opt) {
	    case TCP_MAXSEG:	fprintf(outfp, "TCP_MAXSEG, ");		break;
	    case TCP_NODELAY:	fprintf(outfp, "TCP_NODELAY, ");	break;
	    default:		fprintf(outfp, "TCP_?#%d, ", opt);	break;
	}
	if (get) {
		len = (int *)rd_buf(sysArgs[4], sizeof(int));
		fprintf(outfp, "0x%p, 0x%p=%u) = ",
		    sysArgs[3], sysArgs[4], len);
		free(len);
		return (1);
	}
	fprintf(outfp, "0x%p=", sysArgs[3]);
	val = (int *)rd_buf(sysArgs[3], sizeof(int));
	fprintf(outfp, "%u", *val);
	free(val);
	fprintf(outfp, ", %u) = ", (int)sysArgs[4]);
	return (1);
}
