/* 
 *  print_stat.c -	decode stat/fstat/lstat
 *                      (built/tested under: Digital Unix V3.2)
 *
 *  John Hascall (john@iastate.edu)
 *  Iowa State University Computation Center
 *  01-Mar-1996
 *
 */
#include "trace.h"
#include "sysname.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>

static long	savArgs[2];		/* to show results in exit-rtn */
static int	savNumargs;

static int	printexit_stat();
static void	pr_owners();
static void	pr_mode();

extern FILE *	outfp;
extern int	max_string;

/*  stat(filename, statbufptr) */
/* lstat(filename, statbufptr) */
/* fstat(fd,       statbufptr) */

int printargs_stat (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	savNumargs = sysNumargs;
	savArgs[0] = sysArgs[0];
	savArgs[1] = sysArgs[1];
	switch (sysCallno) {
	    case SYS_stat:
		fprintf(outfp, "stat (");
		p_arg('s', sysArgs[0], 255);
		break;
	    case SYS_lstat:
		fprintf(outfp, "lstat (");
		p_arg('s', sysArgs[0], 255);
		break;
	    case SYS_fstat:
		fprintf(outfp, "fstat (%d", (int)sysArgs[0]);
		break;
	}
	fprintf(outfp, ", 0x%p) = ", sysArgs[1]);
	return (1);
}

int printret_stat (
	long	sysCallno,
	long	sysReturn,
	int	sysErrno
) {
	if (sysReturn == -1) {
		fprintf(outfp, "-1 (%s)\n", sys_errlist[sysErrno]);
		return (1);
	}
	fprintf(outfp, "%d", (int)sysReturn);
	printexit_stat (sysCallno, savArgs, savNumargs, "");
	fprintf(outfp, "\n");
	return (1);
}

/* -------------------- private routines -------------------------- */

static int printexit_stat (
	long	sysCallno,
	long *	sysArgs,
	int	sysNumargs,
	char *	sysFormat
) {
	struct stat *	st;

	fprintf(outfp, " [ , ");
	st = (struct stat *) rd_buf(savArgs[1], sizeof(struct stat));

	fprintf(outfp, "<%u.%u.%u ",
	    major(st->st_dev), minor(st->st_dev), st->st_ino);
	pr_mode(st->st_mode);
	fprintf(outfp, " %u ", st->st_nlink);
	pr_owners(st->st_uid, st->st_gid);
	if ((S_ISBLK(st->st_mode) || S_ISCHR(st->st_mode))) {
		fprintf(outfp, "%u,%u ",
		    major(st->st_rdev), minor(st->st_rdev));
	} else {
		fprintf(outfp, "%u ", st->st_size);
	}
	fprintf(outfp, "%u,%u%,%u", st->st_atime, st->st_mtime, st->st_ctime);
	fprintf(outfp, "> ]");
	free(st);
	return (1);
}

static void pr_owners(
	uid_t	uid,
	gid_t	gid
) {
	struct passwd *	pwd;
	struct group *	grp;

	if ((pwd = getpwuid(uid)) == NULL) {
		fprintf(outfp, "%u ", uid);
	} else {
		fprintf(outfp, "%s ", pwd->pw_name);
	}
	if ((grp = getgrgid(gid)) == NULL) {
		fprintf(outfp, "%u ", gid);
	} else {
		fprintf(outfp, "%s ", grp->gr_name);
	}
}

static void pr_mode (
	mode_t	mode
) {
	char	t	= '?';

	switch (mode & S_IFMT) {
	    case S_IFREG:	t = '-';	break;
	    case S_IFDIR:	t = 'd';	break;
	    case S_IFBLK:	t = 'b';	break;
	    case S_IFCHR:	t = 'c';	break;
	    case S_IFIFO:	t = 'f';	break;
	    case S_IFSOCK:	t = 's';	break;
	    case S_IFLNK:	t = 'l';	break;
	}
	fprintf(outfp, "%c%c%c%c%c%c%c%c%c%c",
	    t,
	    (mode & S_IRUSR) ? 'r' : '-',
	    (mode & S_IWUSR) ? 'w' : '-',
	    (mode & S_IXUSR) ? 'x' : '-',
	    (mode & S_IRGRP) ? 'r' : '-',
	    (mode & S_IWGRP) ? 'w' : '-',
	    (mode & S_IXGRP) ? 'x' : '-',
	    (mode & S_IROTH) ? 'r' : '-',
	    (mode & S_IWOTH) ? 'w' : '-',
	    (mode & S_IXOTH) ? 'x' : '-'
	);
}
