#include "trace.h"
#include "sysname.h"
#include <stdarg.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/times.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdnet/dn.h>
#include <arpa/inet.h>


extern FILE *   outfp;


void pr_flags (
	int	flags,
	...
) {
	va_list	ap;
	int	flag;
	char *	name;
	char *	sep	= "\0\0";

	va_start(ap, flags);
	while ((flag = va_arg(ap, int)) != 0) {
		name = va_arg(ap, char *);
		if (flags & flag) {
			fprintf(outfp, "%s%s", sep, name);
			*sep = '|';
		}
	}
	va_end(ap);
}

void pr_sockaddr (
	struct sockaddr *	s,
	int			len,
	int			bsd44
) {
	unsigned short		family;
	int			i;
	struct sockaddr_dn *	dn;
	struct sockaddr_in *	in;
	struct sockaddr_un *	un;
	static char *		familyname[] = {
		"Unspecified", "UNIX", "INET", "IMP", "PUP", "CHAOS", "NS",
		"ISO", "ECMA", "DATAKIT", "CCITT", "SNA", "DECnet", "DLI",
		"LAT", "HYLINK","APPLETALK", "ROUTE", "LINK", "XTP",
		"NETMAN", "X25", "CTF", "WAN", "USER", "BOGUS"
	};
	/*
	 * deal with stinking evil big-endian-centric BSD 4.4 len hack job
	 */
	if (bsd44) {
		family = (s->sa_family >> 8) & 0x000000ff;
		len = (s->sa_family & 0x000000ff);
	} else {
		family = s->sa_family;
	}

	if (family > AF_MAX) family = AF_MAX;		/* sanity */

	if (bsd44) {
		fprintf(outfp, "<%d, %d/%s, ", len, family, familyname[family]);
	} else {
		fprintf(outfp, "<%d/%s, ", family, familyname[family]);
	}
	switch (family) {
	    case AF_UNIX:
		un = (struct sockaddr_un *)s;
		fprintf(outfp, "%.*s>", len - 2, un->sun_path);
		break;
	    case AF_INET:
		in = (struct sockaddr_in *)s;
		fprintf(outfp, "%u, %s>",
		    ntohs(in->sin_port), inet_ntoa(in->sin_addr));
		break;
	    case AF_DECnet:
		dn = (struct sockaddr_dn *)s;
		fprintf(outfp, "%02.2x, %u, %.*s, ",
		    dn->sdn_flags, dn->sdn_objnum,
		    dn->sdn_objnamel, dn->sdn_objname);
		for (i = 0; i < dn->sdn_add.a_len; ++i) {
			if (i != 0) fprintf(outfp, ".");
			fprintf(outfp, "%u", dn->sdn_add.a_addr[i]);
		}
		fprintf(outfp, ">");
		break;
	    default:
		fprintf(outfp, "...?...>");
	}
}


void pr_timeval (
	struct timeval *	tvp
) {
	if (tvp == NULL) {
		fprintf(outfp, "{}");
	} else {
		tvp = (struct timeval *)rd_buf(tvp, sizeof(struct timeval));
		fprintf(outfp, "{%u,%u}", tvp->tv_sec, tvp->tv_usec);
		free(tvp);
	}
}

